@@ -422,7 +422,8 @@ class _Printer(object):
 def Parse(text,
           message,
           allow_unknown_extension=False,
-          allow_field_number=False):
+          allow_field_number=False,
+          descriptor_pool=None):
   """Parses a text representation of a protocol message into a message.
 
   Args:
@@ -431,6 +432,7 @@ def Parse(text,
     allow_unknown_extension: if True, skip over missing extensions and keep
       parsing
     allow_field_number: if True, both field number and field name are allowed.
+    descriptor_pool: A DescriptorPool used to resolve Any types.
 
   Returns:
     The same message passed as argument.
@@ -440,8 +442,11 @@ def Parse(text,
   """
   if not isinstance(text, str):
     text = text.decode('utf-8')
-  return ParseLines(
-      text.split('\n'), message, allow_unknown_extension, allow_field_number)
+  return ParseLines(text.split('\n'),
+                    message,
+                    allow_unknown_extension,
+                    allow_field_number,
+                    descriptor_pool=descriptor_pool)
 
 
 def Merge(text,
@@ -479,7 +484,8 @@ def Merge(text,
 def ParseLines(lines,
                message,
                allow_unknown_extension=False,
-               allow_field_number=False):
+               allow_field_number=False,
+               descriptor_pool=None):
   """Parses a text representation of a protocol message into a message.
 
   Args:
@@ -496,7 +502,9 @@ def ParseLines(lines,
   Raises:
     ParseError: On text parsing problems.
   """
-  parser = _Parser(allow_unknown_extension, allow_field_number)
+  parser = _Parser(allow_unknown_extension,
+                   allow_field_number,
+                   descriptor_pool=descriptor_pool)
   return parser.ParseLines(lines, message)
 
 
@@ -513,6 +521,7 @@ def MergeLines(lines,
     allow_unknown_extension: if True, skip over missing extensions and keep
       parsing
     allow_field_number: if True, both field number and field name are allowed.
+    descriptor_pool: A DescriptorPool used to resolve Any types.
 
   Returns:
     The same message passed as argument.
@@ -1023,6 +1032,22 @@ class Tokenizer(object):
     self.NextToken()
     return result
 
+  def ConsumeCommentOrTrailingComment(self):
+    """Consumes a comment, returns a 2-tuple (trailing bool, comment str)."""
+
+    # Tokenizer initializes _previous_line and _previous_column to 0. As the
+    # tokenizer starts, it looks like there is a previous token on the line.
+    just_started = self._line == 0 and self._column == 0
+
+    before_parsing = self._previous_line
+    comment = self.ConsumeComment()
+
+    # A trailing comment is a comment on the same line than the previous token.
+    trailing = (self._previous_line == before_parsing
+                and not just_started)
+
+    return trailing, comment
+
   def TryConsumeIdentifier(self):
     try:
       self.ConsumeIdentifier()
